function [A,B] = seccomp(b, varargin)
% SECCOMP   Compute the secular linearization of a matrix polynomial. 
%
%   A = seccomp(b, Pn, ..., P0) computes the secular linearization for the 
%   polynomial specified by
%
%       P(x) = x^n Pn + ... + xP1 + P0
% 
%   given the linear polynomials B_i(x) = (x - b(i)) * eye(m). 
%
%   [A,B] = seccomp(b, Pn, ..., P0), instead, returns the pencil obtained 
%   by using B_n(x) = x*P_n - b(n)*eye(m) in place of (x-b(n))*eye(m). 
%
%   Note: The coefficients of P are taken in reverse order with respect
%   to polyeig. This is done to comply with the order used by Matlab for
%   scalar polynomials, for which this function was originally written. 

m = size(varargin{1}, 1);
P = varargin;

% We need to handle the exeptional case where P is a scalar polynomial. In
% that case, following Matlab standards, the polynomial will be represented
% as a vector, so we should convert it back to our "cell" representation
if min(size(varargin{1})) == 1 && max(size(varargin{1})) > 1
    
    % Check that we have only 1 extra argument in this case
    if nargin ~= 2
        error('Incorrect call to seccomp');        
    end
    
    p = varargin{1};
    
    P = {};
    for i = 1 : length(varargin{1})
        P{i} = p(i);
    end
end

if isa(varargin{1}, 'sym')
    is_symbolic = true;
else
    is_symbolic = false;
end

if nargout == 1
    for i=2:length(P);
        P{i}=P{1}\P{i};
    end
    P{1}=eye(m);

    n = length(P) - 1;
    W = {};

    for i = 1 : n
      x = b(i);
      t = x - b; 
      t = t + (t == 0) ; 
      W{i} = mhorner(x, P{:}) / prod(t);
    end

    WW = [];
    for i = 1 : n
      WW = [ WW W{i} ];
    end
    A = kron(diag(b),eye(m)) - kron(ones(n,1),eye(m)) * WW;
    B = -eye(n*m,n*m);
    
else
    n = length(b);
    
    % Select the values of s
    s = 1;

    I = eye(m);
    e = ones(n,1);
    Imn = eye(m*n);
    z = 0.0;
    
    if is_symbolic
        s = vpa(s);
        I = vpa(I);
        e = vpa(e);
        Imn = vpa(Imn);
        z = vpa(z);
    end
    
    W = cell(1,n);
    
    % Non monic case
    for i = 1 : n-1
        x = b(i); 
        t = x - b(1:n-1);
        t = t + (t == z);
        W{i} = mhorner(x,P{:}) / prod(t);
        W{i} = W{i} / ( (x - b(end))*P{1} + s * I );
    end
    
    % Theorical caveat: this actually works only if W{n} is guaranteed
    % to be a constant and not a higher degree polynomial. 
    W{n} = mhorner(b(n),P{:}) / prod(b(n) - b(1:n-1)) - s * I;
    for i = 1 : n-1
        W{n} = W{n} - s * W{i} / (b(n) - b(i));
    end
    
    WW = [];
    for i = 1 : n
      WW = [ WW W{i} ];
    end
    
    A = kron(diag(b),I); 
    A(end-m+1:end,end-m+1:end) = b(end) * P{1} - s * I;
    A = A - kron(e,I) * WW;
    B = Imn;
    B((n-1)*m+1:n*m,(n-1)*m+1:n*m) = P{1};
    
end

end